use card_backend_pcsc::PcscBackend;
use openpgp_card_sequoia::{state::Open, types::KeyType, Card};
use keyfork_derive_openpgp::openpgp::{Cert, policy::Policy};

/// Factory reset the current card so long as it does not match the last-used backend.
///
/// The return value of `false` means the filter was matched, whereas `true` means it was
/// successfully provisioned.
pub fn factory_reset_current_card(
    card_filter: &mut dyn FnMut(String) -> bool,
    user_pin: &str,
    admin_pin: &str,
    cert: &Cert,
    policy: &dyn Policy,
    card_backend: PcscBackend,
) -> Result<bool, Box<dyn std::error::Error>> {
    let valid_cert = cert.with_policy(policy, None)?;
    let signing_key = valid_cert
        .keys()
        .for_signing()
        .secret()
        .next()
        .expect("no signing key found");
    let decryption_key = valid_cert
        .keys()
        .for_storage_encryption()
        .secret()
        .next()
        .expect("no decryption key found");
    let authentication_key = valid_cert
        .keys()
        .for_authentication()
        .secret()
        .next()
        .expect("no authentication key found");
    let mut card = Card::<Open>::new(card_backend)?;
    let mut transaction = card.transaction()?;
    let application_identifier = transaction.application_identifier()?.ident();
    if !card_filter(application_identifier) {
        return Ok(false);
    }
    transaction.factory_reset()?;
    let mut admin = transaction.to_admin_card("12345678")?;
    admin.upload_key(signing_key, KeyType::Signing, None)?;
    admin.upload_key(decryption_key, KeyType::Decryption, None)?;
    admin.upload_key(authentication_key, KeyType::Authentication, None)?;
    transaction.change_user_pin("123456", user_pin)?;
    transaction.change_admin_pin("12345678", admin_pin)?;
    Ok(true)
}

